@@ -14,64 +14,151 @@ from . import people as cvppl
 
 
 # Specify all externally visible functions this file defines
-__all__ = ['default_sim_plots', 'Sim']
+__all__ = ['default_colors', 'default_sim_plots', 'Sim']
 
+# Specify plot colors
+default_colors = sc.objdict(
+    susceptible = '#5e7544',
+    infectious  = '#c78f65',
+    infections  = '#c75649',
+    tests       = '#aaa8ff',
+    diagnoses   = '#8886cc',
+    recoveries  = '#799956',
+    symptomatic = '#c1ad71',
+    severe      = '#c1981d',
+    critical    = '#b86113',
+    deaths      = '#000000',
+    )
 
 # Specify which quantities to plot -- note, these can be turned on and off by commenting/uncommenting lines
 default_sim_plots = sc.odict({
         'Total counts': sc.odict({
-            'cum_exposed': 'Cumulative infections',
-            'cum_deaths': 'Cumulative deaths',
-            'cum_recoveries':'Cumulative recoveries',
-            # 'cum_tested': 'Cumulative tested',
+            'cum_infections': 'Cumulative infections',
+            'cum_diagnoses':  'Cumulative diagnoses',
+            'cum_recoveries': 'Cumulative recoveries',
+            # 'cum_tests': 'Cumulative tested',
             # 'n_susceptible': 'Number susceptible',
             # 'n_infectious': 'Number of active infections',
-            'cum_diagnosed': 'Cumulative diagnosed',
         }),
         'Daily counts': sc.odict({
-            'infections': 'New infections',
-            'deaths': 'New deaths',
-            'recoveries': 'New recoveries',
+            'new_infections': 'New infections',
+            'new_diagnoses':  'New diagnoses',
+            'new_recoveries': 'New recoveries',
+            'new_deaths':     'New deaths',
             # 'tests': 'Number of tests',
-            'diagnoses': 'New diagnoses',
+        }),
+        'Health outcomes': sc.odict({
+            'cum_severe':   'Cumulative severe cases',
+            'cum_critical': 'Cumulative critical cases',
+            'cum_deaths':   'Cumulative deaths',
+            # 'n_severe':     'Number of severe cases',
+            # 'n_critical':   'Number of critical cases',
         })
-    })
+})
 
 
 class Sim(cvbase.BaseSim):
     '''
     The Sim class handles the running of the simulation: the number of children,
     number of time points, and the parameters of the simulation.
+
+    Args:
+        pars (dict): parameters to modify from their default values
+        datafile (str): filename of (Excel) data file to load, if any
+        datacols (list): list of column names of the data file to load
+        filename (str): the filename for this simulation, if it's saved (default: creation date)
     '''
 
-    def __init__(self, pars=None, datafile=None, filename=None):
+    def __init__(self, pars=None, datafile=None, datacols=None, popfile=None, filename=None):
+        # Create the object
         default_pars = cvpars.make_pars() # Start with default pars
         super().__init__(default_pars) # Initialize and set the parameters as attributes
-        self.datafile = datafile # Store this
-        self.data = None
-        if datafile is not None: # If a data file is provided, load it
-            self.data = cvpars.load_data(datafile)
-        self.created = sc.now()
-        if filename is None:
-            datestr = sc.getdate(obj=self.created, dateformat='%Y-%b-%d_%H.%M.%S')
-            filename = f'covasim_{datestr}.sim'
-        self.filename = filename
-        self.stopped = None # If the simulation has stopped
+
+        # Set attributes
+        self.created       = None  # The datetime the sim was created
+        self.filename      = None  # The filename of the sim
+        self.datafile      = None  # The name of the data file
+        self.data          = None  # The actual data
+        self.popdict       = None  # The population dictionary
+        self.t             = None  # The current time in the simulation
+        self.initialized   = False # Whether or not initialization is complete
+        self.stopped       = None  # If the simulation has stopped
         self.results_ready = False # Whether or not results are ready
-        self.people = {}
-        self.results = {}
-        self.calculated = {}
-        if pars is not None:
-            self.update_pars(pars)
+        self.people        = {}    # Initialize these here so methods that check their length can see they're empty
+        self.results       = {}    # For storing results
+
+        # Now update everything
+        self.set_metadata(filename)        # Set the simulation date and filename
+        self.load_data(datafile, datacols) # Load the data, if provided
+        self.load_population(popfile)      # Load the population, if provided
+        self.update_pars(pars)             # Update the parameters, if provided
         return
 
 
-    def initialize(self):
-        ''' Perform all initializations '''
-        self.validate_pars()
-        self.set_seed()
-        self.init_results()
-        self.init_people()
+    def set_metadata(self, filename):
+        ''' Set the metadata for the simulation -- creation time and filename '''
+        self.created = sc.now()
+        if filename is None:
+            datestr = sc.getdate(obj=self.created, dateformat='%Y-%b-%d_%H.%M.%S')
+            self.filename = f'covasim_{datestr}.sim'
+        return
+
+
+    def load_data(self, datafile=None, datacols=None, **kwargs):
+        ''' Load the data to calibrate against, if provided '''
+        self.datafile = datafile # Store this
+        if datafile is not None: # If a data file is provided, load it
+            self.data = cvpars.load_data(filename=datafile, columns=datacols, **kwargs)
+
+            # Ensure the data are continuous and align with the simulation
+            # data_offset = (self.data.iloc[0]['date'] - self.pars['start_day']).days # TODO: Use df.set_index("A").reindex(new_index).reset_index()
+
+        return
+
+
+    def load_population(self, filename=None, **kwargs):
+        '''
+        Load the population dictionary from file.
+
+        Args:
+            filename (str): name of the file to load
+        '''
+        if filename is not None:
+            filepath = sc.makefilepath(filename=filename, **kwargs)
+            self.popdict = sc.loadobj(filepath)
+            n_actual = len(self.popdict['uid'])
+            n_expected = self['n']
+            if n_actual != n_expected:
+                errormsg = f'Wrong number of people ({n_expected} requested, {n_actual} actual) -- please change "n" to match or regenerate the file'
+                raise ValueError(errormsg)
+        return
+
+
+    def save_population(self, filename, **kwargs):
+        '''
+        Save the population dictionary to file.
+
+        Args:
+            filename (str): name of the file to save to.
+        '''
+        filepath = sc.makefilepath(filename=filename, **kwargs)
+        sc.saveobj(filepath, self.popdict)
+        return filepath
+
+
+    def initialize(self, **kwargs):
+        '''
+        Perform all initializations.
+
+        Args:
+            kwargs (dict): passed to init_people
+        '''
+        self.t = None # The current time index
+        self.validate_pars() # Ensure parameters have valid values
+        self.set_seed() # Reset the random seed
+        self.init_results() # Create the results stucture
+        self.init_people(**kwargs) # Create all the people (slow)
+        self.initialized = True
         return
 
 
@@ -87,7 +174,7 @@ class Sim(cvbase.BaseSim):
         self['start_day'] = start_day # Convert back
 
         # Handle population data
-        popdata_choices = ['random', 'bayesian', 'data']
+        popdata_choices = ['random', 'microstructure']
         if sc.isnumber(self['usepopdata']) or isinstance(self['usepopdata'], bool): # Convert e.g. usepopdata=1 to 'bayesian'
             self['usepopdata'] = popdata_choices[int(self['usepopdata'])] # Choose one of these
         if self['usepopdata'] not in popdata_choices:
@@ -103,40 +190,47 @@ class Sim(cvbase.BaseSim):
 
 
     def init_results(self):
-        ''' Initialize results '''
+        '''
+        Create the main results structure.
+        We differentiate between flows, stocks, and cumulative results
+        The prefix "new" is used for flow variables, i.e. counting new events (infections/deaths/recoveries) on each timestep
+        The prefix "n" is used for stock variables, i.e. counting the total number in any given state (sus/inf/rec/etc) on any paticular timestep
+        The prefix "cum_" is used for cumulative variables, i.e. counting the total number that have ever been in a given state at some point in the sim
+        Note that, by definition, n_dead is the same as cum_deaths and n_recovered is the same as cum_recoveries, so we only define the cumulative versions
+        '''
 
         def init_res(*args, **kwargs):
             ''' Initialize a single result object '''
             output = cvbase.Result(*args, **kwargs, npts=self.npts)
             return output
 
-        # Create the main results structure
-        self.results['n_susceptible']  = init_res('Number susceptible')
-        self.results['n_exposed']      = init_res('Number exposed')
-        self.results['n_infectious']   = init_res('Number infectious')
-        self.results['n_symptomatic']  = init_res('Number symptomatic')
-        self.results['n_recovered']    = init_res('Number recovered')
-        self.results['infections']     = init_res('Number of new infections')
-        self.results['tests']          = init_res('Number of tests')
-        self.results['diagnoses']      = init_res('Number of new diagnoses')
-        self.results['recoveries']     = init_res('Number of new recoveries')
-        self.results['deaths']         = init_res('Number of new deaths')
-        self.results['cum_exposed']    = init_res('Cumulative number exposed')
-        self.results['cum_tested']     = init_res('Cumulative number of tests')
-        self.results['cum_diagnosed']  = init_res('Cumulative number diagnosed')
-        self.results['cum_deaths']     = init_res('Cumulative number of deaths')
-        self.results['cum_recoveries'] = init_res('Cumulative number recovered')
-        self.results['doubling_time']  = init_res('Doubling time', scale=False)
-        self.results['r_eff']          = init_res('Effective reproductive number', scale=False)
+        dcols = default_colors # Shorten
+
+        # Stock variables
+        self.results['n_susceptible'] = init_res('Number susceptible',       color=dcols.susceptible)
+        self.results['n_exposed']     = init_res('Number exposed',           color=dcols.infections)
+        self.results['n_infectious']  = init_res('Number infectious',        color=dcols.infectious)
+        self.results['n_symptomatic'] = init_res('Number symptomatic',       color=dcols.symptomatic)
+        self.results['n_severe']      = init_res('Number of severe cases',   color=dcols.severe)
+        self.results['n_critical']    = init_res('Number of critical cases', color=dcols.critical)
+        self.results['bed_capacity']  = init_res('Percentage bed capacity', scale=False)
+
+        # Flows and cumulative flows
+        self.result_flows = ['infections', 'tests', 'diagnoses', 'recoveries', 'symptomatic', 'severe', 'critical', 'deaths']
+        for key in self.result_flows:
+            suffix = ' cases' if key in ['symptomatic', 'severe', 'critial'] else '' # Since need to say "severe cases" not just "severe"
+            self.results[f'new_{key}'] = init_res(f'Number of new {key}{suffix}', color=dcols[key]) # Flow variables -- e.g. "Number of new infections"
+            self.results[f'cum_{key}'] = init_res(f'Cumulative {key}{suffix}',    color=dcols[key]) # Cumulative variables -- e.g. "Cumulative infections"
+
+        # Other variables
+        self.results['r_eff']         = init_res('Effective reproductive number', scale=False)
+        self.results['doubling_time'] = init_res('Doubling time', scale=False)
 
         # Populate the rest of the results
         self.results['t'] = self.tvec
         self.results['date'] = [self['start_day'] + dt.timedelta(days=int(t)) for t in self.tvec]
         self.results_ready = False
 
-        # Create calculated values structure
-        self.calculated['eff_beta'] = (1-self['default_symp_prob'])*self['asym_factor']*self['beta'] + self['default_symp_prob']*self['beta']  # Using asymptomatic proportion
-        self.calculated['r_0']      = self['contacts']*self['dur']*self.calculated['eff_beta']
         return
 
 
@@ -151,14 +245,15 @@ class Sim(cvbase.BaseSim):
         return res_keys
 
 
-    def init_people(self, verbose=None, id_len=None):
+    def init_people(self, verbose=None, id_len=None, **kwargs):
         ''' Create the people '''
+
         if verbose is None:
             verbose = self['verbose']
 
         sc.printv(f'Creating {self["n"]} people...', 1, verbose)
 
-        cvppl.make_people(self, verbose=verbose, id_len=id_len)
+        cvppl.make_people(self, verbose=verbose, id_len=id_len, **kwargs)
 
         # Create the seed infections
         for i in range(int(self['n_infected'])):
@@ -168,12 +263,60 @@ class Sim(cvbase.BaseSim):
         return
 
 
-    def run(self, initialize=True, do_plot=False, verbose=None, **kwargs):
-        ''' Run the simulation '''
+    def next(self, steps=None, stop=None, initialize=False, finalize=False, verbose=0, **kwargs):
+        '''
+        A method to step through the simulation rather than run all at once.
+
+        Args:
+            steps (int): the number of timesteps to run (default: 1)
+            stop (int): alternative to steps, index of the simulation to run until (default: current + 1)
+            kwargs (dict): passed to self.run()
+
+        Returns:
+            None.
+        '''
+        if self.t is None:
+            start = 0
+        else:
+            start = self.t + 1
+        if steps is None:
+            steps = 1
+        if stop is None:
+            stop = start + steps
+        self.run(start=start, stop=stop, initialize=initialize, finalize=finalize, verbose=verbose, **kwargs)
+        return None # Unlike run(), do not return results
+
+
+
+    def run(self, start=None, stop=None, initialize=None, finalize=None, do_plot=False, verbose=None, **kwargs):
+        '''
+        Run the simulation.
+
+        Args:
+            start (int): when to start the simulation relative to the time vector; default 0
+            stop (int): when to stop; default -1
+            initialize (bool): whether to initialize people and results objects
+            finalize (bool): whether or not to calculate final results objects after the time loop
+            do_plot (bool): whether to plot
+            verbose (int): level of detail to print
+            kwargs (dict): passed to self.plot()
+
+        Returns:
+            results: the results object (also modifies in-place)
+        '''
 
         T = sc.tic()
 
         # Reset settings and results
+        if start is None:
+            start = 0
+        if initialize is None:
+            if start > 0 or self.initialized:
+                initialize = False
+            else:
+                initialize = True
+        if finalize is None:
+            finalize = True
         if verbose is None:
             verbose = self['verbose']
         if initialize:
@@ -181,7 +324,9 @@ class Sim(cvbase.BaseSim):
 
         # Main simulation loop
         self.stopped = False # We've just been asked to run, so ensure we're unstopped
-        for t in range(self.npts):
+        tvec = self.tvec[start:stop]
+        for t in tvec:
+            self.t = t # Store the current time
 
             # Check timing and stopping function
             elapsed = sc.toc(T, output=True)
@@ -190,29 +335,39 @@ class Sim(cvbase.BaseSim):
                 self.stopped = {'why':'timelimit', 'message':'Time limit exceeded at step {t}', 't':t}
 
             if self['stop_func']:
-                self.stopped = self['stop_func'](self, t) # Feed in the current simulation object and the time
+                self.stopped = self['stop_func'](self) # Feed in the current simulation object and the time
 
             # If this gets set, stop running -- e.g. if the time limit is exceeded
             if self.stopped:
                 break
 
-            # Zero counts for this time step.
-            n_susceptible = 0
-            n_exposed     = 0
-            n_deaths      = 0
-            n_recoveries  = 0
-            n_infectious  = 0
-            n_infections  = 0
-            n_symptomatic = 0
-            n_recovered   = 0
+            # Zero counts for this time step: stocks
+            n_susceptible   = 0
+            n_exposed       = 0
+            n_infectious    = 0
+            n_symptomatic   = 0
+            n_severe        = 0
+            n_critical      = 0
+
+            # Zero counts for this time step: flows
+            new_recoveries  = 0
+            new_deaths      = 0
+            new_infections  = 0
+            new_symptomatic = 0
+            new_severe      = 0
+            new_critical    = 0
 
             # Extract these for later use. The values do not change in the person loop and the dictionary lookup is expensive.
             rand_popdata     = (self['usepopdata'] == 'random')
             beta             = self['beta']
-            asym_factor      = self['asym_factor']
+            asymp_factor     = self['asymp_factor']
             diag_factor      = self['diag_factor']
             cont_factor      = self['cont_factor']
             beta_pop         = self['beta_pop']
+            n_beds           = self['n_beds']
+            bed_constraint   = False
+            n_people         = len(self.people)
+            n_comm_contacts  = self['contacts_pop']['R'] # TODO: refactor name
 
             # Print progress
             if verbose>=1:
@@ -222,7 +377,6 @@ class Sim(cvbase.BaseSim):
                 else:
                     print(string)
 
-
             # Update each person, skipping people who are susceptible
             not_susceptible = filter(lambda p: not p.susceptible, self.people.values())
             n_susceptible = len(self.people)
@@ -233,31 +387,36 @@ class Sim(cvbase.BaseSim):
                 # If exposed, check if the person becomes infectious or develops symptoms
                 if person.exposed:
                     n_exposed += 1
-                    if not person.infectious and t >= person.date_infectious: # It's the day they become infectious
+                    if not person.infectious and t == person.date_infectious: # It's the day they become infectious
                         person.infectious = True
                         sc.printv(f'      Person {person.uid} became infectious!', 2, verbose)
-                    if not person.symptomatic and person.date_symptomatic is not None and t >= person.date_symptomatic:  # It's the day they develop symptoms
-                        person.symptomatic = True
-                        sc.printv(f'      Person {person.uid} developed symptoms!', 2, verbose)
 
                 # If infectious, check if anyone gets infected
                 if person.infectious:
 
-                    # Check for death
-                    died = person.check_death(t)
-                    n_deaths += died
+                    # Check whether the person died on this timestep
+                    new_death = person.check_death(t)
+                    new_deaths += new_death
 
-                    # Check for recovery
-                    recovered = person.check_recovery(t)
-                    n_recoveries += recovered
+                    # Check whether the person recovered on this timestep
+                    new_recovery = person.check_recovery(t)
+                    new_recoveries += new_recovery
+
+                    # No recovery: check symptoms
+                    if not new_recovery:
+                        new_symptomatic, n_symptomatic = person.check_symptomatic(t, new_symptomatic, n_symptomatic)
+                        new_severe,      n_severe      = person.check_severe(t,      new_severe,      n_severe)
+                        new_critical,    n_critical    = person.check_critical(t,    new_critical,    n_critical)
+                        if n_severe > n_beds:
+                            bed_constraint = True
 
                     # If the person didn't die or recover, check for onward transmission
-                    if not died and not recovered:
+                    if not new_death and not new_recovery:
                         n_infectious += 1 # Count this person as infectious
 
                         # Calculate transmission risk based on whether they're asymptomatic/diagnosed/have been isolated
                         thisbeta = beta * \
-                                   (asym_factor if person.symptomatic else 1.) * \
+                                   (asymp_factor if person.symptomatic else 1.) * \
                                    (diag_factor if person.diagnosed else 1.) * \
                                    (cont_factor if person.known_contact else 1.)
 
@@ -266,6 +425,8 @@ class Sim(cvbase.BaseSim):
                             transmission_inds = cvu.bf(thisbeta, person.contacts)
                         else: # Dictionary of contacts -- extra loop over layers
                             transmission_inds = []
+                            community_contact_inds = cvu.choose(max_n=n_people, n=n_comm_contacts)
+                            person.contacts['R'] = community_contact_inds
                             for ckey in self.contact_keys:
                                 layer_beta = thisbeta * beta_pop[ckey]
                                 transmission_inds.extend(cvu.bf(layer_beta, person.contacts[ckey]))
@@ -280,44 +441,49 @@ class Sim(cvbase.BaseSim):
 
                             # Skip people who are not susceptible
                             if target_person.susceptible:
-                                n_infections += target_person.infect(t, person) # Actually infect them
+                                new_infections += target_person.infect(t, bed_constraint, source=person) # Actually infect them
                                 sc.printv(f'        Person {person.uid} infected person {target_person.uid}!', 2, verbose)
 
 
-                # Count people who developed symptoms
-                if person.symptomatic:
-                    n_symptomatic += 1
-
-                # Count people who recovered
-                if person.recovered:
-                    n_recovered += 1
-
+            sc.printv(f'Number of beds available: {n_beds-n_severe}, bed constraint: {bed_constraint}', 2, verbose)
             # End of person loop; apply interventions
             for intervention in self['interventions']:
-                intervention.apply(self, t)
+                intervention.apply(self)
             if self['interv_func'] is not None: # Apply custom intervention function
-                self =self['interv_func'](self, t)
+                self =self['interv_func'](self)
 
-            # Update counts for this time step
-            self.results['n_susceptible'][t] = n_susceptible
-            self.results['n_exposed'][t]     = n_exposed
-            self.results['deaths'][t]        = n_deaths
-            self.results['recoveries'][t]    = n_recoveries
-            self.results['n_infectious'][t]  = n_infectious
-            self.results['infections'][t]    = n_infections
-            self.results['n_symptomatic'][t] = n_symptomatic
-            self.results['n_recovered'][t]   = n_recovered
+            # Update counts for this time step: stocks
+            self.results['n_susceptible'][t]  = n_susceptible
+            self.results['n_exposed'][t]      = n_exposed
+            self.results['n_infectious'][t]   = n_infectious # Tracks total number infectious at this timestep
+            self.results['n_symptomatic'][t]  = n_symptomatic # Tracks total number symptomatic at this timestep
+            self.results['n_severe'][t]       = n_severe # Tracks total number of severe cases at this timestep
+            self.results['n_critical'][t]     = n_critical # Tracks total number of critical cases at this timestep
+            self.results['bed_capacity'][t]   = n_severe/n_beds if n_beds>0 else None
+
+            # Update counts for this time step: flows
+            self.results['new_infections'][t]  = new_infections # New infections on this timestep
+            self.results['new_recoveries'][t]  = new_recoveries # Tracks new recoveries on this timestep
+            self.results['new_symptomatic'][t] = new_symptomatic
+            self.results['new_severe'][t]      = new_severe
+            self.results['new_critical'][t]    = new_critical
+            self.results['new_deaths'][t]      = new_deaths
 
         # End of time loop; compute cumulative results outside of the time loop
-        self.results['cum_exposed'].values    = pl.cumsum(self.results['infections'].values) + self['n_infected'] # Include initially infected people
-        self.results['cum_tested'].values     = pl.cumsum(self.results['tests'].values)
-        self.results['cum_diagnosed'].values  = pl.cumsum(self.results['diagnoses'].values)
-        self.results['cum_deaths'].values     = pl.cumsum(self.results['deaths'].values)
-        self.results['cum_recoveries'].values = pl.cumsum(self.results['recoveries'].values)
+        if finalize:
+            self.finalize(verbose=verbose) # Finalize the results
+            sc.printv(f'\nRun finished after {elapsed:0.1f} s.\n', 1, verbose)
+            self.summary = self.summary_stats(verbose=verbose)
+            if do_plot: # Optionally plot
+                self.plot(**kwargs)
 
-        # Add in the results from the interventions
-        for intervention in self['interventions']:
-            intervention.finalize(self)  # Execute any post-processing
+        return self.results
+
+
+    def finalize(self, verbose=None):
+        for key in self.result_flows:
+            self.results[f'cum_{key}'].values = np.cumsum(self.results[f'new_{key}'].values)
+        self.results['cum_infections'].values += self['n_infected'] # Include initially infected people
 
         # Scale the results
         for reskey in self.reskeys:
@@ -329,19 +495,12 @@ class Sim(cvbase.BaseSim):
         self.compute_r_eff()
         self.likelihood()
 
-        # Tidy up
-        self.results_ready = True
-        sc.printv(f'\nRun finished after {elapsed:0.1f} s.\n', 1, verbose)
-        self.results['summary'] = self.summary_stats()
-
-        if do_plot:
-            self.plot(**kwargs)
-
         # Convert to an odict to allow e.g. sim.people[25] later, and results to an objdict to allow e.g. sim.results.diagnoses
         self.people = sc.odict(self.people)
         self.results = sc.objdict(self.results)
+        self.results_ready = True
 
-        return self.results
+        return
 
 
     def compute_doubling(self, window=None, max_doubling_time=100):
@@ -360,7 +519,7 @@ class Sim(cvbase.BaseSim):
             None (modifies results in place)
         '''
         window = self['window']
-        cum_infections = self.results['cum_exposed']
+        cum_infections = self.results['cum_infections'].values
         for t in range(window, self.npts):
             infections_now = cum_infections[t]
             infections_prev = cum_infections[t-window]
@@ -402,7 +561,7 @@ class Sim(cvbase.BaseSim):
         return
 
 
-    def likelihood(self, verbose=None):
+    def likelihood(self, weights=None, verbose=None):
         '''
         Compute the log-likelihood of the current simulation based on the number
         of new diagnoses.
@@ -410,15 +569,25 @@ class Sim(cvbase.BaseSim):
         if verbose is None:
             verbose = self['verbose']
 
+        if weights is None:
+            weights = {}
+
         loglike = 0
-        if self.data is not None and len(self.data): # Only perform likelihood calculation if data are available
-            for d,datum in enumerate(self.data['new_positives']):
-                if not pl.isnan(datum): # Skip days when no tests were performed
-                    estimate = self.results['diagnoses'][d]
-                    p = cvu.poisson_test(datum, estimate)
-                    logp = pl.log(p)
-                    loglike += logp
-                    sc.printv(f'  {self.data["date"][d]}, data={datum:3.0f}, model={estimate:3.0f}, log(p)={logp:10.4f}, loglike={loglike:10.4f}', 2, verbose)
+        if self.data is not None: # Only perform likelihood calculation if data are available
+            for key in self.reskeys:
+                if key in self.data:
+                    if key in weights:
+                        weight = weights[key]
+                    else:
+                        weight = 1
+                    for d,datum in enumerate(self.data[key]):
+                        if not pl.isnan(datum) and d<len(self.results[key].values):
+                            estimate = self.results[key][d]
+                            if datum and estimate:
+                                p = cvu.poisson_test(datum, estimate)
+                                logp = pl.log(p)
+                                loglike += weight*logp
+                                sc.printv(f'  {self.data["date"][d]}, data={datum:3.0f}, model={estimate:3.0f}, log(p)={logp:10.4f}, loglike={loglike:10.4f}', 2, verbose)
 
             self.results['likelihood'] = loglike
             sc.printv(f'Likelihood: {loglike}', 1, verbose)
@@ -433,18 +602,12 @@ class Sim(cvbase.BaseSim):
             verbose = self['verbose']
 
         summary = sc.objdict()
+        summary_str = 'Summary:\n'
         for key in self.reskeys:
             summary[key] = self.results[key][-1]
-
-        sc.printv(f"""Summary:
-     {summary['n_susceptible']:5.0f} susceptible
-     {summary['n_infectious']:5.0f} infectious
-     {summary['n_symptomatic']:5.0f} symptomatic
-     {summary['cum_exposed']:5.0f} total exposed
-     {summary['cum_diagnosed']:5.0f} total diagnosed
-     {summary['cum_deaths']:5.0f} total deaths
-     {summary['cum_recoveries']:5.0f} total recovered
-               """, 1, verbose)
+            if key.startswith('cum_'):
+                summary_str += f'   {summary[key]:5.0f} {self.results[key].name}\n'
+        sc.printv(summary_str, 1, verbose)
 
         return summary
 
@@ -487,9 +650,9 @@ class Sim(cvbase.BaseSim):
         to_plot = sc.odict(to_plot) # In case it's supplied as a dict
 
         # Handle input arguments -- merge user input with defaults
-        fig_args     = sc.mergedicts({'figsize':(16,12)}, fig_args)
+        fig_args     = sc.mergedicts({'figsize':(16,14)}, fig_args)
         plot_args    = sc.mergedicts({'lw':3, 'alpha':0.7}, plot_args)
-        scatter_args = sc.mergedicts({'s':150, 'marker':'s'}, scatter_args)
+        scatter_args = sc.mergedicts({'s':70, 'marker':'s'}, scatter_args)
         axis_args    = sc.mergedicts({'left':0.1, 'bottom':0.05, 'right':0.9, 'top':0.97, 'wspace':0.2, 'hspace':0.25}, axis_args)
 
         fig = pl.figure(**fig_args)
@@ -501,30 +664,14 @@ class Sim(cvbase.BaseSim):
         res = self.results # Shorten since heavily used
 
         # Plot everything
-
-        colors = sc.gridcolors(max([len(tp) for tp in to_plot.values()]))
-
-        # Define the data mapping. Must be here since uses functions
-        if self.data is not None and len(self.data):
-            data_mapping = {
-                'cum_exposed': pl.cumsum(self.data['new_infections']),
-                'cum_diagnosed':  pl.cumsum(self.data['new_positives']),
-                'cum_tested':     pl.cumsum(self.data['new_tests']),
-                'infections':     self.data['new_infections'],
-                'tests':          self.data['new_tests'],
-                'diagnoses':      self.data['new_positives'],
-                }
-        else:
-            data_mapping = {}
-
         for p,title,keylabels in to_plot.enumitems():
-            ax = pl.subplot(2,1,p+1)
+            ax = pl.subplot(len(to_plot),1,p+1)
             for i,key,label in keylabels.enumitems():
-                this_color = colors[i]
+                this_color = res[key].color
                 y = res[key].values
                 pl.plot(res['t'], y, label=label, **plot_args, c=this_color)
-                if key in data_mapping:
-                    pl.scatter(self.data['day'], data_mapping[key], c=[this_color], **scatter_args)
+                if self.data is not None and key in self.data:
+                    pl.scatter(self.data['day'], self.data[key], c=[this_color], **scatter_args)
             if self.data is not None and len(self.data):
                 pl.scatter(pl.nan, pl.nan, c=[(0,0,0)], label='Data', **scatter_args)
 
@@ -565,3 +712,29 @@ class Sim(cvbase.BaseSim):
             pl.close(fig)
 
         return fig
+
+
+    def plot_result(self, key, fig_args=None, plot_args=None):
+        '''
+        Simple method to plot a single result. Useful for results that aren't
+        standard outputs.
+
+        Args:
+            key (str): the key of the result to plot
+            fig_args (dict): passed to pl.figure()
+            plot_args (dict): passed to pl.plot()
+
+        Example:
+            sim.plot_result('doubling_time')
+        '''
+        fig_args  = sc.mergedicts({'figsize':(16,10)}, fig_args)
+        plot_args = sc.mergedicts({'lw':3, 'alpha':0.7}, plot_args)
+        fig = pl.figure(**fig_args)
+        pl.subplot(111)
+        tvec = self.results['t']
+        res = self.results[key]
+        y = res.values
+        color = res.color
+        pl.plot(tvec, y, c=color, **plot_args)
+        return fig
+
